@@ -2,16 +2,19 @@
 
 import os
 import re
-import tempfile
+from pathlib import Path
 
 from .log import logger
 
-VERSION = "2.1.5"
+VERSION = "2.1.7"
 
-TEMP_DIR = tempfile.gettempdir()  # str type
+CACHE_DIR = Path.home() / ".cache" / "fakeua"
+CACHE_DIR.mkdir(parents=True, exist_ok=True)
+DB = str(CACHE_DIR / "fake_useragent_{version}.json".format(version=VERSION))
 
 
-def find_tempfile(dir):
+def get_cache(dir):
+    logger.debug(f"Got cache folder: {dir}")
     for _, _, files in os.walk(dir):
         for f in files:
             match = re.search(r"^fake_useragent_", f)
@@ -23,10 +26,6 @@ def find_tempfile(dir):
     return ""
 
 
-DB = os.path.join(
-    TEMP_DIR, "fake_useragent_{version}.json".format(version=VERSION)
-)  # str type
-
 BROWSER_BASE_PAGE = (
     "http://useragentstring.com/pages/useragentstring.php?name={browser}"
 )
