@@ -22,44 +22,39 @@ from functools import wraps
 import asyncio
 from aiohttp import ClientSession
 
-
 # sys.path.append(os.path.dirname(os.path.dirname(os.path.abspath(__file__))))
 # from fake_user_agent import settings
 # from fake_user_agent.log import logger
-# from fake_user_agent.errors import FakeUserAgentError
-# from fake_user_agent.parse import parse_args
+# from fake_user_agent.args import parse_args
 
 from . import settings
 from .log import logger
-from .errors import FakeUserAgentError
-from .parse import parse_args
+from .args import parse_args
 
 
 all_versions = defaultdict(list)  # a dict created with its values being list
 
 OP = ["FETCHING", "PARSING"]
 
-TEMP_FILE = ""  # if `TEMP_FILE` is imported from settings.py, later logger level change won't affect `find_tempfile` that has been run immediately after importing
+CACHE_FILE = ""
 
 
 async def fetch(url, session):
     """Fetch html text file using aiohttp session."""
 
     attempt = 0
-    headers = {
-        "User-Agent": "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.62 Safari/537.36"
-    }
+    headers = {"User-Agent": "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.62 Safari/537.36"}
 
     while True:
         try:
-            async with session.get(
-                url, headers=headers, timeout=settings.HTTP_TIMEOUT, ssl=False
-            ) as resp:
+            async with session.get(url, headers=headers, timeout=settings.HTTP_TIMEOUT, ssl=False) as resp:
                 result = await resp.text()
         except asyncio.TimeoutError as e:
             attempt = call_on_error(e, url, attempt, OP[0])
+            continue
         except Exception as e:
             attempt = call_on_error(e, url, attempt, OP[0])
+            continue
         else:
             logger.debug(f"{url} has been fetched successfully")
             return result
@@ -69,12 +64,7 @@ def call_on_error(error, url, attempt, op):
     """Retry mechanism when an error occurs."""
 
     attempt += 1
-    logger.debug(
-        "%s HTML from %s %d times",
-        op,
-        url,
-        attempt,
-    )
+    logger.debug("%s HTML from %s %d times", op, url, attempt)
     if attempt == 3:
         logger.debug("Maximum %s retries reached. Exit", op)
         logger.error(str(error))
@@ -90,19 +80,17 @@ async def parse(browser, session):
     lxml_element = etree.HTML(html_str)
 
     attempt = 0
-
     while True:
         try:
-            versions = lxml_element.xpath('//*[@id="liste"]/ul/li/a/text()')[
-                : settings.BROWSERS_COUNT_LIMIT
-            ]
+            # data type of the list:  <class 'lxml.etree._ElementUnicodeResult'>
+            versions = lxml_element.xpath('//*[@id="liste"]/ul/li/a/text()')[:settings.BROWSERS_COUNT_LIMIT]
         except Exception as e:
             attempt = call_on_error(e, url, attempt, OP[1])
+            continue
         else:
             if not versions:
-                attempt = call_on_error(
-                    FakeUserAgentError("Nothing parsed out"), url, attempt, OP[1]
-                )
+                attempt = call_on_error(ValueError("Nothing parsed out"), url, attempt, OP[1])
+                continue
 
             logger.debug(f"{browser} has been parsed successfully")
             return versions
@@ -113,14 +101,12 @@ async def write_to_dict(browser, session):
 
     global all_versions
     versions = await parse(browser, session)
-    all_versions[browser].extend(
-        versions
-    )  # add each element of versions list to the end of the list to be extended
+    all_versions[browser].extend(versions)  # add each element of versions list to the end of the list to be extended
     logger.debug(f"{browser} versions has been written to all_versions")
 
 
 def write(path, data):
-    """Write a json tempfile as cache if there isn't one, or update it if any."""
+    """Write a json cache if there isn't one, or update it if any."""
 
     with open(path, encoding="utf-8", mode="wt") as f:
         dumped = json.dumps(data)
@@ -128,8 +114,8 @@ def write(path, data):
 
     logger.debug(f"Cache has been stored in {path}\n")
 
-    global TEMP_FILE
-    TEMP_FILE = path
+    global CACHE_FILE
+    CACHE_FILE = path
 
 
 def read(path):
@@ -142,28 +128,28 @@ def read(path):
     return json.loads(cache_data)
 
 
-def rm_tempfile():
-    """Remove the tempfile as cache if any."""
+def rm_cache():
+    """Remove cache if any."""
 
-    tempfile = settings.find_tempfile(settings.TEMP_DIR)
-    if not tempfile:
-        print("No tempfile found to be deleted.")
+    cache = settings.get_cache(settings.CACHE_DIR)
+    if not cache:
+        print("No cache found to be deleted.")
         return
 
-    os.remove(tempfile)
+    os.remove(cache)
 
-    file_name = tempfile.split("/")[-1]
+    file_name = cache.split("/")[-1]
     print(f"{file_name} has been removed successfully.")
 
-    global TEMP_FILE
-    TEMP_FILE = ""
+    global CACHE_FILE
+    CACHE_FILE = ""
 
 
 def get_browser(browser):
     """If browser name is not given, randomly choose one browser based on weights set in settings.BROWSERS."""
 
     if not browser:
-        logger.debug(f"A browser will be randowly given")
+        logger.debug("A browser will be randowly given")
         browser = random.choices(
             list(settings.BROWSERS.keys()),
             weights=list(settings.BROWSERS.values()),
@@ -172,14 +158,13 @@ def get_browser(browser):
 
     else:
         logger.debug(f"{browser} will be formatted")
-        if not isinstance(browser, str):
-            raise FakeUserAgentError("Browser name must be string.")
         browser = browser.strip().lower()
         browser = settings.SHORTCUTS.get(browser, browser)
-        if browser not in list(
-            settings.BROWSERS.keys()
-        ):  # transform an iterator to a list
-            raise FakeUserAgentError(f"{browser} is not supported.")
+        if browser not in list(settings.BROWSERS.keys()):  # transform an iterator to a list
+            print(f"Browser name '{browser}' is not supported.")
+            print("Supported browsers: chrome, edge, firefox, safari, opera")
+            print("\nPlease try again with one of the supported browser names.")
+            sys.exit()
 
     return browser
 
@@ -192,28 +177,30 @@ def timer(func):
         start_at = time.time()
         f = func(*args, **kwargs)
         time_taken = round((time.time() - start_at), 4)
-        print("Time taken: {} seconds".format(time_taken))
+        print("\nTime taken: {} seconds".format(time_taken))
         return f
 
     return wrapper
 
 
-async def main(browser=None, use_tempfile=True):
+async def main(browser=None, use_cache=True):
     browser = get_browser(browser)
     logger.debug(f"Got {browser}")
 
-    if not use_tempfile:
+    if not use_cache:
         async with ClientSession() as session:
             versions = await parse(browser, session)
             return random.choice(versions)
-    else:
-        tempfile = settings.find_tempfile(settings.TEMP_DIR)
-        global TEMP_FILE
-        TEMP_FILE = tempfile
 
-        if TEMP_FILE:
-            data = read(TEMP_FILE)
+    else:
+        cache = settings.get_cache(settings.CACHE_DIR)
+        global CACHE_FILE
+        CACHE_FILE = cache
+
+        if CACHE_FILE:
+            data = read(CACHE_FILE)
             return random.choice(data[browser])
+
         else:
             async with ClientSession() as session:
                 tasks = []
@@ -240,7 +227,7 @@ def get_input():
             sys.exit()
 
         if args.remove:
-            rm_tempfile()
+            rm_cache()
             sys.exit()
 
         if args.debug:
@@ -249,18 +236,18 @@ def get_input():
         browser = args.browser
 
         if args.nocache:
-            print(asyncio.run(main(browser=browser, use_tempfile=False)))
+            print(asyncio.run(main(browser=browser, use_cache=False)))
         else:
-            print(asyncio.run(main(browser=browser, use_tempfile=True)))
+            print(asyncio.run(main(browser=browser, use_cache=True)))
 
     except KeyboardInterrupt:
         print("\nStopped by user.")
 
 
-def user_agent(browser=None, use_tempfile=True):
+def user_agent(browser=None, use_cache=True):
     """Entry point for getting a user agent by importing this function in a python script."""
 
-    return asyncio.run(main(browser, use_tempfile))
+    return asyncio.run(main(browser, use_cache))
 
 
 if __name__ == "__main__":
